<?php
/**
 * Created by PhpStorm.
 * User: Administrator
 * Date: 2019/6/3
 * Time: 14:36
 */

namespace App\Http\Controllers;


use App\Repositories\MetaRepository;
use App\Transformers\MetaTransformer;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use League\Fractal\Manager;
use League\Fractal\Resource\Collection;
use League\Fractal\Pagination\IlluminatePaginatorAdapter;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use SimpleSoftwareIO\QrCode\Facades\QrCode;

class MetaController extends Controller
{
    public function __construct(MetaRepository $metaRepository)
    {
        $this->metaRepository = $metaRepository;
    }

    public function index(Request $request)
    {
        $productList = $this->metaRepository->index($request->all());
        $fractal = new Manager();
        $resource = new Collection($productList, new MetaTransformer());
        $resource->setPaginator(new IlluminatePaginatorAdapter($productList));
        $data = $fractal->createData($resource)->toArray();
        $data['extra'] = [
            'filters' => [
                'patch_num',
            ],
            'columns' => [
                'patch_num',
                'md5',
                'zip',
                'status',
                'trans_id',
                'created_at',
                'updated_at',
            ]
        ];
        return $data;
    }

    /**
     * 发布信息
     */
    public function create(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'patch_num' => 'required|alpha_num|max:16',
            'produce_date' => 'required|date',
            'variety' => 'required|max:4',
            'variety_img' => 'required|url',
            'age' => 'required|max:4',
            'food' => 'required|max:5',
            'food_img' => 'required|url',
            'water' => 'required|max:5',
            'water_img' => 'required|url',
            'farm.*' => 'required|url',
            'report' => 'required'
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return  $this->metaRepository->create($request->all());
    }
}